/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/
#pragma once
#ifndef DATAFOX_COMMUNICATION_MODULE_TABLE_H
#define DATAFOX_COMMUNICATION_MODULE_TABLE_H

// Basisklasse einbinden.
#include "datafox/communication/module_application.h"

// MODULE_TABLE( 3 )

#define IV_MODULE_TABLE                          0x0003

// Fehler:
// 1 -> Modul unbekannt
// 2 -> Funktion unbekannt
// 3 -> Zu wenige Parameter
// 4 -> Zu viele Parameter

// Get: Datensatz-/Listenbeschreibung. GET_TABLE( 0 )
#define IV_FUNCTION_TABLE_GET                    0x0000
// (Majorname.Minorname\0)
// (STable2)

// Die Tabellenbeschreibung kann auf folgende Arten ermittelt werden.
// Zum einen ist es noetig die Datensatz-/Listenbeschreibungen des Setup sowie der
// Zutrittskontrolle auseinanderzuhalten. Dann wird entweder der Index oder der Name,
// der jeweilingen Tabellendefinition benoetigt.

// Majornamen:
// RECORD - Sind die Datensatzbeschreibungen
// LIST   - Sind die Listenbeschreibungen
// ACCESS - Sind die Listenbeschreibungen der Zutrittskontrolle

// darauf folgt nach einem Punkt entweder die Nummer oder der Name der Tabelle.

// Get: Datensatz-/Listenbeschreibungsfeld. GET_FIELD( 1 )
#define IV_FUNCTION_TABLE_GET_COLUMN             0x0001
// (Majorname.Minorname.Subminorname\0)
// (STableField)

// Majorname.Minorname bauen sich wie bei den Tabellen auf. Der Subminorname entspricht
// in seinem Aufbau dem Minornamen, ist logischerweise jedoch Feldbezogen.

// Fehler:
// 1 = MAJOR_NAME ist unbekannt
// 2 = MINOR_NAME ist unbekannt
// 3 = SUBMINOR_NAME ist unbekannt

#define IV_FUNCTION_TABLE_OPEN                         0x0002
//params[in]
// 1. char *name;
// 2. char *reserved;
//params[out]
// 1. int32_t handle;
// 2. char *reserved;

#define IV_FUNCTION_TABLE_CLOSE                        0x0003
// params[int32_t]
// 1. int32_t handle;
// params[out]
// -

#define IV_FUNCTION_TABLE_SET_FILTER                   0x0004
// params[int32_t]
// 1. int32_t handle;
// 2. char *filter;
// params[out]
// -
//-> Aktive Antwort SET_FILTER_OK erwartet.

#define IV_FUNCTION_TABLE_SET_FILTER_OK                0x0005
// params[int32_t]
// -
// params[out]
// -

#define IV_FUNCTION_TABLE_GET_FILTER                   0x0006
// params[int32_t]
// 1. int32_t handle;
// params[out]
// 1. char *filter;

#define IV_FUNCTION_TABLE_REMOVE_FILTER                0x0007
// params[int32_t]
// 1. int32_t handle;
// params[out]
// -

#define IV_FUNCTION_TABLE_SET_CURRENT_ROW              0x0008
// params[int32_t]
// 1. int32_t handle;
// 2. int32_t offset;
// 3. uint16_t direction;
// params[out]
// -

#define IV_FUNCTION_TABLE_SET_CURRENT_COLUMN_DATA_1    0x0009
// params[int32_t]
// 1. int32_t handle;
// 2. char *name;
// 3. uint8_t *data;
// params[out]
// -

#define IV_FUNCTION_TABLE_GET_CURRENT_COLUMN_DATA_1    0x000A
// params[int32_t]
// 1. int32_t handle;
// 2. char *name;
// params[out]
// 1. uint8_t *data;

#define IV_FUNCTION_TABLE_SET_CURRENT_ROW_DATA_1       0x000B
// params[int32_t]
// 1. int32_t handle;
// 2. unsinged char *data;
// params[out]
// -

#define IV_FUNCTION_TABLE_GET_CURRENT_ROW_DATA_1       0x000C
// params[int32_t]
// 1. int32_t handle;
// params[out]
// 1. unsinged char *data;

#define IV_FUNCTION_TABLE_GENERATE                     0x000D
// params[int32_t]
// 1. char *name;
// 2. char *reserved;
// params[out]
// -
//-> Aktive-Antwort TABLE_GENERATE_OK erwartet.

#define IV_FUCNTION_TABLE_GENERATE_OK                  0x000E
// params[int32_t]
// -
// params[out]
// -

#define IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD    0x000F
// params[int32_t]
// 1. uint8_t type;
// 2. uint8_t *data;
// params[out]
// -
// -> Aktive Antwort APPEND_RECORD_OK erwartet.

#define IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK 0x0010
// params[int32_t]
// -
// params[out]
// -

#define IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD1      0x0011
// DEPRECATED: Unterstuetzte keine Aktive-Antwort.

#define IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD1   0x0012
// DEPRECATED: Unterstuetzte keine Aktive-Antwort.

#define IV_FUNCTION_TABLE_GET_ROW_COUNT                0x0013
// params[int32_t]
// 1. int32_t handle;
// params[out]
// 1. uint32_t currentCount;
// 2. uint32_t unsortedCount;
// 3. uint32_t deletedCount;

#define IV_FUNCTION_TABLE_GET_CURRENT_ROW              0x0014
// params[int32_t]
// 1. int32_t handle;
// params[out]
// 1. uint32_t pos;

#define IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD       0x0015
// params[int32_t]
// 1. uint8_t type;
// 2. uint32_t pid;
// 3. uint8_t fid;
// params[out]
// -
//-> Aktive-Antwort FINGERPRINT_TABLE_GET_RECORD_OK erwartet.

#define IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD_OK    0x0016
// params[int32_t]
// 1. uint8_t type;
// 2. uint8_t *data;
// params[out]
// -

#define IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD    0x0017
// params[int32_t]
// 1. uint32_t pid;
// 2. uint8_t fid;
// params[out]
// -
//-> Aktive-Antwort FINGERPRINT_TABLE_DELETE_RECORD_OK erwartet.

#define IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD_OK 0x0018
// params[int32_t]
// -
// params[out]
// -

#define IV_FUNCTION_TABLE_APPEND_ROW_DATA              0x0019
// params[int32_t]
// 1. int32_t handle;
// 2. uint16_t flags;
// 3. char separator;
// 4. unsinged char *data;
// params[out]
// -

#define IV_FUNCTION_TABLE_DELETE_CURRENT_ROW           0x001A
// params[int32_t]
// 1. int32_t handle;
// 2. uint16_t flags;
// params[out]
// -

#define IV_FUNCTION_TABLE_SET_CURRENT_ROW_DATA         0x001B
// params[int32_t]
// 1. int32_t handle;
// 2. uint16_t flags;
// 3. char separator;
// 4. unsinged char *data;
// params[out]
// -

#define IV_FUNCTION_TABLE_GET_CURRENT_ROW_DATA         0x001C
// params[int32_t]
// 1. int32_t handle;
// 2. uint16_t flags;
// 3. char separator;
// params[out]
// 1. unsinged char *data;

#define IV_FUNCTION_TABLE_SET_CURRENT_COLUMN_DATA      0x001D
// params[int32_t]
// 1. int32_t handle;
// 2. uint16_t flags;
// 3. char *name;
// 4. uint8_t *data;
// params[out]
// -

#define IV_FUNCTION_TABLE_GET_CURRENT_COLUMN_DATA      0x001E
// params[int32_t]
// 1. int32_t handle;
// 2. uint16_t flags;
// 3. char *name;
// params[out]
// 1. uint8_t *data;

#define IV_FUNCTION_TABLE_SET_ALL_ROWS_TO_COLUMN_DATA  0x001F
// params[int32_t]
// 1. int32_t handle;
// 2. uint16_t flags;
// 3. char *name;
// 4. uint8_t *data;
// params[out]
// -
//-> Aktive-Antwort FINGERPRINT_TABLE_SET_ALL_ROWS_TO_COLUMN_DATA_OK erwartet.

#define IV_FUNCTION_TABLE_SET_ALL_ROWS_TO_COLUMN_DATA_OK   0x0020

#define IV_FUNCTION_TABLE_DELETE_AVAILABLE_ROWS            0x0021
// params[int32_t]
// 1. int32_t handle;
// 2. uint16_t flags;
// params[out]
// -
//-> Aktive-Antwort IV_FUNCTION_TABLE_DELETE_AVAILABLE_ROWS_OK erwartet.

#define IV_FUNCTION_TABLE_DELETE_AVAILABLE_ROWS_OK         0x0022

namespace Datafox
{
    namespace Communication
    {
        class ModuleTable : public ModuleApplication
        {
        public:
            /// Konstruktor fuer Initialisierung der Klasse.
            ModuleTable( int32_t connectionNumber = ConnectionBase::defaultConnectionNumber(), uint8_t deviceAddress = ProtocolBase::defaultDeviceAddress() );

            /// Destruktor
            virtual ~ModuleTable();

        public:
            // Gibt Modulfehler zurueck.
            virtual void errorModule( uint16_t module, uint16_t error, std::string &text );
            // Gibt Funktionsfehlertext zurueck.
            virtual void errorFunction( uint16_t module, uint16_t func, uint16_t error, std::string &text );

            // MODULE_TABLE

            // Tabellendefinition ueber Index oder Namen ermitteln.
            int32_t readTableStruct( uint8_t deviceAddress, const std::string &name, StructTable2 &table );
            // Tabellenfelddefinition ueber Index oder Namen ermitteln.
            int32_t readTableColumnStruct( uint8_t deviceAddress, const std::string &name, StructTable2Field &field );

            // Oeffnen einer Tabelle
            int32_t openTable( uint8_t deviceAddress, const std::string &name, uint32_t options, int32_t &handle, std::string &reserved );
            // Schliessen der Tabelle
            int32_t closeTable( uint8_t deviceAddress, int32_t handle );

            // Setzen eines Filters auf die Tabelle
            int32_t writeTableFilter( uint8_t deviceAddress, int32_t handle, const std::string &filter );
            int32_t replyWriteTableFilter( uint8_t deviceAddress );
            // Lesen des Filters auf die Tabelle
            int32_t readTableFilter( uint8_t deviceAddress, int32_t handle, std::string &filter );
            // Entfernen des Filters
            int32_t removeTableFilter( uint8_t deviceAddress, int32_t handle );
    
            // Abfragen der Zeilenanzahl
            int32_t readTableRowCount( uint8_t deviceAddress, int32_t handle, uint32_t &currentCount, uint32_t &unsortedCount, uint32_t &deletedCount );
            // Abfragen der aktuellen Datensatznummer
            int32_t readCurrentTableRow( uint8_t deviceAddress, int32_t handle, uint32_t &currentRow );
            // Ueberspringen von n Datensaetzen (offset kann negativ sein), ab top/aktuell/bottom
            int32_t writeCurrentTableRow( uint8_t deviceAddress, int32_t handle, int32_t offset, uint16_t direction );
    
            // Aendern eines aktuellen Datensatzes
            int32_t writeCurrentTableRowData( uint8_t deviceAddress, int32_t handle, const char *data, uint16_t dataSize );
            int32_t writeCurrentTableRowDataExt( uint8_t deviceAddress, int32_t handle, uint16_t flags, char separator, const char *data, uint16_t dataSize );
            // Aendern eines Spaltenwerts, des aktuellen Datensatzes
            int32_t writeCurrentTableColumnData( uint8_t deviceAddress, int32_t handle, const char *name, const char *data );
            int32_t writeCurrentTableColumnDataExt( uint8_t deviceAddress, int32_t handle, uint16_t flags, const char *name, const char *data, uint16_t dataSize );
            int32_t writeAllTableRowsColumnData( uint8_t deviceAddress, int32_t handle, uint16_t flags, const char *name, const char *data, uint16_t dataSize );
            int32_t replyWriteAllTableRowsColumnData( uint8_t deviceAddress );
    
            // Abfragen eines aktuellen Datensatzes
            int32_t readCurrentTableRowData( uint8_t deviceAddress, int32_t handle, char *data, uint16_t &dataSize );
            int32_t readCurrentTableRowDataExt( uint8_t deviceAddress, int32_t handle, uint16_t flags, char separator, char *data, uint16_t &dataSize );
            // Abfragen eines Spaltenwerts, des aktuellen Datensatzes
            int32_t readCurrentTableColumnData( uint8_t deviceAddress, int32_t handle, const char *name, char *data, uint16_t &dataSize );
            int32_t readCurrentTableColumnDataExt( uint8_t deviceAddress, int32_t handle, uint16_t flags, const char *name, char *data, uint16_t &dataSize );

            // Daten an Listentabelle anfuegen.
            int32_t appendTableRowData( uint8_t deviceAddress, int32_t handle, uint16_t flags, char separator, const char *data, uint16_t dataSize );
            // Loeschen des aktuellen Datensatzes
            int32_t deleteCurrentTableRow( uint8_t deviceAddress, int32_t handle, uint16_t flags );
            // Loeschen der Datensaetze die durch den gesetzten Filter erreichbar sind.
            int32_t deleteFiltratedTableRows( uint8_t deviceAddress, int32_t handle, uint16_t flags );
            int32_t replyDeleteFiltratedTableRows( uint8_t deviceAddress );

            // Tabelle erzeugen
            int32_t generateTable( uint8_t deviceAddress, const std::string &name, const std::string &reserved );
            int32_t replyGenerateTable( uint8_t deviceAddress );

            // Datensatz an Fingerprint-Tabelle anfuegen.
            int32_t appendFingerprintRecord( uint8_t deviceAddress, uint8_t type, const uint8_t *data, uint16_t size );
            int32_t replyAppendFingerprintRecord( uint8_t deviceAddress );
            // Datensatz aus Fingerprint-Tabelle lesen.
            int32_t readFingerprintRecord( uint8_t deviceAddress, uint8_t &type, uint32_t pid, uint8_t fid, uint8_t *data, uint16_t &size );
            int32_t replyReadFingerprintRecord( uint8_t deviceAddress );
            // Datensatz aus Fingerprint-Tabelle loeschen.
            int32_t deleteFingerprintRecord( uint8_t deviceAddress, uint32_t pid, uint8_t fid );
            int32_t replyDeleteFingerprintRecord( uint8_t deviceAddress );

            //#################################
            // CallRoutinen

            // Ausfuehren von Befehlen
            virtual int32_t remoteCall( uint16_t module, uint16_t func, uint8_t *data, uint16_t &count );

        };

    } // namespace Communication
} // namespace Datafox

#endif
